#!/usr/local/bin/ruby
# $Id: response_bench.rb,v 1.4 2002/01/14 03:46:34 toki Exp $
# -
# wwwsrv's response benchmark

require 'benchmark'		# -> http://www.notwork.org/~gotoken/ruby/p/benchmark/
require 'wwwsrv/logger'
require 'wwwsrv/request'
require 'wwwsrv/response'
require 'wwwsrv/document'
require 'wwwsrv/server'

include Benchmark
STDOUT.sync = true

trial = (ARGV.first && ARGV.first.to_i) || 1_000

def comma(num)
  num.to_s.reverse.scan(/\d\d?\d?/).join(',').reverse
end

pseudo_output = Object::new
def pseudo_output.write(msg); self; end
def pseudo_output.flush; self; end

sid = 'A.1-1'
prefix = ''
srv_opt = WWWsrv::ServerOption::new
srv_opt.log = WWWsrv::Logger::new(pseudo_output)
host = WWWsrv::Host::new('localhost', 8088)
request = WWWsrv::Request::new('GET', '/', WWWsrv::HTTP_1_1, WWWsrv::Header::new)
request.host = host
request.header.accept = 'image/gif, image/x-xbitmap, image/jpeg, image/pjpeg, */*'
request.header.accept_language = 'ja'
request.header.accept_encoding = 'gzip, deflate'
request.header.user_agent = 'Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 5.0)'
request.header.host = 'localhost:8088'
request.header.connection = 'Keep-Alive'
response = WWWsrv::Response::new

puts "[WWWsrv::Page\#content #{comma(trial)}]"
page = WWWsrv::Page::new("Hello world\n", 'text/plain', Time::now)
puts(measure{
  trial.times do
    page.content(sid, srv_opt, prefix, request, response) do |arg, *opts|
      # nothing to do.
    end
  end
})

puts "[WWWsrv::ResponseWrite\#write * #{comma(trial)}]"
watcher = WWWsrv::ResponseWatcher::new
puts(measure{
  trial.times do
    res_writer = WWWsrv::ResponseWriter::new(pseudo_output, request, watcher)
    page.content(sid, srv_opt, prefix, request, response) do |arg, *opts|
      res_writer.write(sid, arg, *opts)
    end
    res_writer.terminate(sid)
  end
})

puts "[WWWsrv::WriteBuffer\#write * #{comma(trial)}]"
wbuf = WWWsrv::WriteBuffer::new(pseudo_output, WWWsrv::Server::TIMEOUT, WWWsrv::Server::BUFFER_SIZE)
puts(measure{
  trial.times do
    res_writer = WWWsrv::ResponseWriter::new(wbuf, request, watcher)
    page.content(sid, srv_opt, prefix, request, response) do |arg, *opts|
      res_writer.write(sid, arg, *opts)
    end
    res_writer.terminate(sid)
  end
})
