#!/usr/local/bin/ruby
# $Id: pseudo_doc.rb,v 1.1 2001/11/17 14:11:12 toki Exp $
# -
# Pseudo Document for WWW server test

require 'wwwsrv'
require 'comm'
require 'pseudo_io'
require 'runit/testcase'
require 'runit/cui/testrunner'

module TestWWWsrv
  class PseudoDocument < WWWsrv::Document
    def initialize(content_type, content_fragments)
      @content_type = content_type
      @content = content_fragments
      @length = nil
    end

    attr_reader :content_type

    def set_content_length
      @length = 0
      for content_fragment in @content
	@length += content_fragment.length
      end

      self
    end

    def content(sid, srv_opt, prefix, request, response)
      srv_opt.log.debug("sid #{sid}: debug #{type}")
      response.header.content_type = @content_type
      response.header.content_length = @length.to_s if @length
      yield(response)
      for content_fragment in @content
	yield(content_fragment)
      end

      self
    end
  end

  class TestPseudoDocument < RUNIT::TestCase
    include Communicate

    def setup
      @content = [
	'01234',
	'56789',
	'ABCDE'
      ]
      @document = PseudoDocument::new('text/plain', @content)
      @request = WWWsrv::Request::new('GET', '/', WWWsrv::HTTP_1_1)
      @request.header = WWWsrv::Header::new
      @request.header.host = 'localhost'
      @request.header.user_agent = "Test/#{WWWsrv::VERSION} (with RUNIT)"
      @request.content = PseudoIO::new
      @request.host = WWWsrv::Host::new('localhost')
      @request.client = WWWsrv::Client::new('localhost', '127.0.0.1')
      @log_iobuf = PseudoIO::new
      @srv_opt = WWWsrv::ServerOption::new
      @srv_opt.root = @document
      @srv_opt.log = WWWsrv::Logger::new(@log_iobuf, WWWsrv::Logger::DEBUG)
    end

    def test_content
      response = WWWsrv::Response::new
      state, err_args, doc_response, content_spool, content_alist =
	communicate(@document, 'A.1-1', @srv_opt, '', @request, response)
      assert_equal(:success, state)
      assert_equal(200, doc_response.status)
      assert_equal('text/plain', doc_response.header.content_type)
      assert_nil(doc_response.header.content_length)
      assert_equal(@content.join, content_spool)
      assert_equal(@content, content_alist.map{ |fragment,*opts| fragment })
    end

    def test_content_with_length
      @document.set_content_length
      response = WWWsrv::Response::new
      state, err_args, doc_response, content_spool, content_alist =
	communicate(@document, 'A.1-1', @srv_opt, '', @request, response)
      assert_equal(:success, state)
      assert_equal(200, doc_response.status)
      assert_equal('text/plain', doc_response.header.content_type)
      assert_equal(@content.join.length, doc_response.header.content_length.to_i)
      assert_equal(@content.join, content_spool)
      assert_equal(@content, content_alist.map{ |fragment,*opts| fragment })
    end
  end
end

if ($0 == __FILE__) then
  RUNIT::CUI::TestRunner::run(TestWWWsrv::TestPseudoDocument::suite)
end
