#!/usr/local/bin/ruby
# $Id: test_alert.rb,v 1.1 2002/01/09 13:55:23 toki Exp $
# -
# Test of WWWsrv::AlertDocument class
# Test of WWWsrv::DebugAlertDocument class

require 'comm'
require 'pseudo_io'
require 'pseudo_doc'
require 'wwwsrv/logger'
require 'wwwsrv/request'
require 'wwwsrv/response'
require 'wwwsrv/alert'
require 'runit/testcase'
require 'runit/cui/testrunner'

module TestWWWsrv
  class TestAlertDocument < RUNIT::TestCase
    def setup
      @alert = WWWsrv::AlertDocument::new
      @request = WWWsrv::Request::new('GET', '/', WWWsrv::HTTP_1_0)
      @request.header = WWWsrv::Header::new
      @request.header.host = 'localhost'
      @request.header.user_agent = 'Test/0.9 (with RUNIT)'
      @request.content = PseudoIO::new
      @request.host = WWWsrv::Host::new('localhost')
      @request.client = WWWsrv::Client::new('localhost', '127.0.0.1')
      @log_iobuf = PseudoIO::new
      @srv_opt = WWWsrv::ServerOption::new
      @srv_opt.root = PseudoDocument::new('text/plain', [ "This is test\n" ])
      @srv_opt.log = WWWsrv::Logger::new(@log_iobuf, WWWsrv::Logger::DEBUG)
    end

    def test_new
      assert_instance_of(WWWsrv::AlertDocument, @alert)
    end

    def test_content
      alert_page = ''
      alert_response = nil
      response = WWWsrv::Response::new(WWWsrv::HTTP_1_0, 404)
      @srv_opt.error = 'test'
      @alert.content('1-1', @srv_opt, '', @request, response) do |arg, *opts|
	unless (alert_response) then
	  alert_response = arg
	else
	  alert_page.concat(arg)
	end
      end

      assert_instance_of(WWWsrv::Response, alert_response)
      assert_equal('text/html', alert_response.header.content_type)
      assert(! alert_page.empty?)
    end
  end

  class TestDebugAlertDocument < RUNIT::TestCase
    def setup
      @alert = WWWsrv::DebugAlertDocument::new
      @request = WWWsrv::Request::new('GET', '/', WWWsrv::HTTP_1_0)
      @request.header = WWWsrv::Header::new
      @request.header.host = 'localhost'
      @request.header.user_agent = 'Test/0.9 (with RUNIT)'
      @request.content = PseudoIO::new
      @request.host = WWWsrv::Host::new('localhost')
      @request.client = WWWsrv::Client::new('localhost', '127.0.0.1')
      @log_iobuf = PseudoIO::new
      @srv_opt = WWWsrv::ServerOption::new
      @srv_opt.root = WWWsrv::Document::new
      @srv_opt.log = WWWsrv::Logger::new(@log_iobuf, WWWsrv::Logger::DEBUG)
    end

    def test_new
      assert_instance_of(WWWsrv::DebugAlertDocument, @alert)
    end

    def test_content
      alert_page = ''
      alert_response = nil
      response = WWWsrv::Response::new(WWWsrv::HTTP_1_0, 404)
      @srv_opt.error = 'test'
      @alert.content('1-1', @srv_opt, '', @request, response) do |arg, *opts|
	unless (alert_response) then
	  alert_response = arg
	else
	  alert_page.concat(arg)
	end
      end

      assert_instance_of(WWWsrv::Response, alert_response)
      assert_equal('text/html', alert_response.header.content_type)
      assert(! alert_page.empty?)
    end
  end
end

if ($0 == __FILE__) then
  TestWWWsrv::constants.map{ |name|
    TestWWWsrv::const_get(name)
  }.select{ |const|
    (const.kind_of? Class) && (const < RUNIT::TestCase)
  }.each do |test_case|
    RUNIT::CUI::TestRunner::run(test_case::suite)
  end
end
