#!/usr/local/bin/ruby
# $Id: test_chunk.rb,v 1.4 2002/01/06 15:45:48 toki Exp $
# -
# Test of WWWsrv::ChunkedTransferCoding module

require 'pseudo_io'
require 'wwwsrv/chunk'
require 'runit/testcase'
require 'runit/cui/testrunner'

module TestWWWsrv
  class TestChunkedTransferCoding < RUNIT::TestCase
    def setup
      @input = PseudoIO::new
    end

    def test_parse_chunk
      data = 'Hello world.'
      @input.write(format("%x\r\n", data.length))
      @input.write(data)
      @input.write("\r\n")
      chunk_data, chunk_extension = WWWsrv::ChunkedTransferCoding::parse_chunk(@input)
      assert_equal(data, chunk_data)
      assert(chunk_extension.empty?)
    end

    def test_parse_chunk_extentsion
      data = 'Hello world.'
      @input.write(format('%x', data.length) + '; debug; test=true' + "\r\n")
      @input.write(data)
      @input.write("\r\n")
      chunk_data, chunk_extension = WWWsrv::ChunkedTransferCoding::parse_chunk(@input)
      assert_equal(data, chunk_data)
      assert_not_nil(chunk_extension.assoc('debug'))
      assert_nil(chunk_extension.assoc('debug')[1])
      assert_not_nil(chunk_extension.assoc('test'))
      assert_equal(chunk_extension.assoc('test')[1], 'true')
    end

    def test_parse_chunk_ln
      data = 'Hello world.'
      @input.write(format("%x\n", data.length))
      @input.write(data)
      @input.write("\n")
      chunk_data, chunk_extension = WWWsrv::ChunkedTransferCoding::parse_chunk(@input)
      assert_equal(data, chunk_data)
      assert(chunk_extension.empty?)
    end

    def test_parse_chunk_bad_size_format
      data = 'Hello world.'
      @input.write("XXX\r\n")
      @input.write(data)
      @input.write("\r\n")
      assert_exception(WWWsrv::ChunkParseError) { WWWsrv::ChunkedTransferCoding::parse_chunk(@input) }
    end

    def test_parse_chunk_short_data
      data = 'Hello world.'
      @input.write(format("%x\r\n", data.length))
      @input.write(data.split.first)
      @input.write("\r\n")
      assert_exception(WWWsrv::ChunkParseError) { WWWsrv::ChunkedTransferCoding::parse_chunk(@input) }
    end

    def test_parse_chunk_long_data
      data = 'Hello world.'
      @input.write(format("%x\r\n", data.length))
      @input.write(data)
      @input.write('x')
      assert_exception(WWWsrv::ChunkParseError) { WWWsrv::ChunkedTransferCoding::parse_chunk(@input) }
    end
  end
end

if ($0 == __FILE__) then
  TestWWWsrv::constants.map{ |name|
    TestWWWsrv::const_get(name)
  }.select{ |const|
    (const.kind_of? Class) && (const < RUNIT::TestCase)
  }.each do |test_case|
    RUNIT::CUI::TestRunner::run(test_case::suite)
  end
end
