#!/usr/local/bin/ruby
# $Id: test_filter.rb,v 1.4 2001/11/17 14:11:15 toki Exp $
# -
# Test of document filter for WWW server

require 'wwwsrv'
require 'wwwsrv/filter'
require 'runit/testcase'
require 'runit/cui/testrunner'
require 'comm'
require 'pseudo_io'

module TestWWWsrv
  class TestDocumentFilter < RUNIT::TestCase
    include Communicate

    def setup
      @name = 'index.html'
      @content = <<-EOF
<html>
<head>
<title>Test</title>
</head>
<body>
<h1>Test</h1>
<p>Hello world.</p>
</body>
</html>
      EOF

      @root = WWWsrv::DocumentFolder::new
      @root.mount(WWWsrv::Page::new(@content), '/', @name)

      @log_iobuf = PseudoIO::new
      @srv_opt = WWWsrv::ServerOption::new
      @srv_opt.root = @root
      @srv_opt.log = WWWsrv::Logger::new(@log_iobuf, WWWsrv::Logger::DEBUG)

      @request = WWWsrv::Request::new
      @request.version = WWWsrv::HTTP_1_1
      @request.header = WWWsrv::Header::new
      @request.header.host = 'localhost'
      @request.header.user_agent = 'Test/0.9 (with RUNIT)'
      @request.host = WWWsrv::Host::new('localhost')
      @request.content = PseudoIO::new
      @request.client = WWWsrv::Client::new('localhost', '127.0.0.1')
    end

    def test_through
      doc_filter = WWWsrv::DocumentFilter::new(@root) { |line| line }
      @request.method = 'GET'
      @request.path = '/index.html'
      response = WWWsrv::Response::new

      state, err_args, doc_response, doc_content, doc_content_alist =
	communicate(doc_filter, '1-1', @srv_opt, '', @request, response)

      assert_equal(:success, state)
      assert_nil(doc_response.header.content_length)
      assert_equal(@content, doc_content)
      assert(! doc_content_alist.empty?)
    end

    def test_modify
      doc_filter = WWWsrv::DocumentFilter::new(@root) { |line|
	line.gsub(/Hello/, 'Bye bye')
      }
      @request.method = 'GET'
      @request.path = '/index.html'
      response = WWWsrv::Response::new

      state, err_args, doc_response, doc_content, doc_content_alist =
	communicate(doc_filter, '1-1', @srv_opt, '', @request, response)

      assert_equal(:success, state)
      assert_nil(doc_response.header.content_length)
      assert_equal(@content.gsub(/Hello/, 'Bye bye'), doc_content)
      assert(! doc_content_alist.empty?)
    end

    def test_no_new_line
      @content = <<-EOF
<html>
<head>
<title>Test</title>
</head>
<body>
<h1>Test</h1>
      EOF
      @content.concat("<p>Hello world.</p>\n" * 1024)
      @content.concat(<<-EOF)
</body>
</html>
      EOF
      @content.chomp!
      @root.mount(WWWsrv::Page::new(@content), '/', @name)

      doc_filter = WWWsrv::DocumentFilter::new(@root) { |line|
	line.gsub(/Hello/, 'Bye bye')
      }
      @request.method = 'GET'
      @request.path = '/index.html'
      response = WWWsrv::Response::new

      state, err_args, doc_response, doc_content, doc_content_alist =
        communicate(doc_filter, '1-1', @srv_opt, '', @request, response)

      assert_equal(:success, state)
      assert_nil(doc_response.header.content_length)
      assert_equal(@content.gsub(/Hello/, 'Bye bye'), doc_content)
      assert(! doc_content_alist.empty?)
    end
  end
end

if ($0 == __FILE__) then
  RUNIT::CUI::TestRunner::run(TestWWWsrv::TestDocumentFilter::suite)
end
