#!/usr/local/bin/ruby
# $Id: test_form.rb,v 1.4 2002/01/06 15:45:50 toki Exp $
# -
# Test of WWWsrv::FormData module

require 'pseudo_io'
require 'wwwsrv/htmlconv'
require 'wwwsrv/form'
require 'runit/testcase'
require 'runit/cui/testrunner'

module TestWWWsrv
  class TestFormData < RUNIT::TestCase
    def test_parse_query
      param_data = [
	%w[ double first ],
	%w[ double second ],
	%w[ debug true ]
      ].map{ |param_name, param_value|
	WWWsrv::HTMLconv::encodeURL(param_name) + '=' +
	  WWWsrv::HTMLconv::encodeURL(param_value)
      }.join('&')

      params = WWWsrv::FormData::parse_query(param_data)
      assert_equal('first', params['double'][0])
      assert_equal('second', params['double'][1])
      assert_equal('true', params['debug'][0])
    end

    def make_multipart_input(src)
      input = PseudoIO::new
      src.each do |line|
 	line.chomp!("\n")
 	line.chomp!("\r")
 	input.write(line + "\r\n")
      end

      input
    end
    private :make_multipart_input

    def test_parse_form_data
      content_type = 'multipart/form-data; boundary=AaB03x'
      content_reader = make_multipart_input(<<-EOF)
--AaB03x
Content-Disposition: form-data; name="submit"

OK
--AaB03x
Content-Disposition: form-data; name="date"

1975-11-19
--AaB03x
Content-Disposition: form-data; name="files"
Content-Type: multipart/mixed; boundary="Bb:C04y"

--Bb:C04y
Content-Disposition: file; filename="hello.html"
Content-Type: text/html

<html>
<body><p>Hello world.</p></body>
</html>

--Bb:C04y
Content-Disposition: file; filename="hello.css"
Content-Type: text/css

p { font-weight: bold; font-size: large }

--Bb:C04y--
--AaB03x--
      EOF

      form_data = WWWsrv::FormData::parse_form_data(content_reader, content_type)

      header, body = form_data['submit'].shift
      assert_equal('form-data; name="submit"', header.content_disposition)
      assert_equal('OK', body)

      header, body = form_data['date'].shift
      assert_equal('form-data; name="date"', header.content_disposition)
      assert_equal('1975-11-19', body)

      header, submultipart = form_data['files'].shift
      assert_equal('form-data; name="files"', header.content_disposition)
      assert_equal('multipart/mixed; boundary="Bb:C04y"', header.content_type)

      header, body = submultipart.shift
      assert_equal('file; filename="hello.html"', header.content_disposition)
      assert_equal('text/html', header.content_type)
      assert_equal("<html>\r\n<body><p>Hello world.</p></body>\r\n</html>\r\n", body)

      header, body = submultipart.shift
      assert_equal('file; filename="hello.css"', header.content_disposition)
      assert_equal('text/css', header.content_type)
      assert_equal("p { font-weight: bold; font-size: large }\r\n", body)

      assert(submultipart.empty?)
      form_data.each_key do |name|
	assert(form_data[name].empty?)
      end
    end
  end
end

if ($0 == __FILE__) then
  TestWWWsrv::constants.map{ |name|
    TestWWWsrv::const_get(name)
  }.select{ |const|
    (const.kind_of? Class) && (const < RUNIT::TestCase)
  }.each do |test_case|
    RUNIT::CUI::TestRunner::run(test_case::suite)
  end
end
