#!/usr/local/bin/ruby
# $Id: test_logger.rb,v 1.3 2002/01/13 01:12:24 toki Exp $
# -
# Test of WWWsrv::Logger class
# Test of WWWsrv::MultiLogger class

require 'pseudo_io'
require 'wwwsrv/logger'
require 'runit/testcase'
require 'runit/cui/testrunner'

module TestWWWsrv
  class TestLogger < RUNIT::TestCase
    def setup
      @dbg_iobuf = PseudoIO::new
      @dbg_logger = WWWsrv::Logger::new(@dbg_iobuf, WWWsrv::Logger::DEBUG)
      @dsc_iobuf = PseudoIO::new
      @dsc_logger = WWWsrv::Logger::new(@dsc_iobuf, WWWsrv::Logger::SYSTEM)
      @msg_iobuf = PseudoIO::new
      @msg_logger = WWWsrv::Logger::new(@msg_iobuf, WWWsrv::Logger::MESSAGE)
      @wrn_iobuf = PseudoIO::new
      @wrn_logger = WWWsrv::Logger::new(@wrn_iobuf, WWWsrv::Logger::WARNING)
      @alt_iobuf = PseudoIO::new
      @alt_logger = WWWsrv::Logger::new(@alt_iobuf, WWWsrv::Logger::ALERT)
      @err_iobuf = PseudoIO::new
      @err_logger = WWWsrv::Logger::new(@err_iobuf, WWWsrv::Logger::ERROR)
    end

    def test_new
      assert_instance_of(WWWsrv::Logger, @dbg_logger)
      assert_instance_of(WWWsrv::Logger, @msg_logger)
      assert_instance_of(WWWsrv::Logger, @wrn_logger)
      assert_instance_of(WWWsrv::Logger, @alt_logger)
      assert_instance_of(WWWsrv::Logger, @err_logger)
    end

    def test_debug
      @dbg_logger.print(WWWsrv::Logger::DEBUG, 'hello')
      assert_match(/hello/, @dbg_iobuf.data)
      @dbg_logger.debug('world')
      assert_match(/world/, @dbg_iobuf.data)
      @dsc_logger.print(WWWsrv::Logger::DEBUG, 'hello')
      assert_nil(@dsc_iobuf.data)
      @dsc_logger.debug('world')
      assert_nil(@dsc_iobuf.data)
      @msg_logger.print(WWWsrv::Logger::DEBUG, 'hello')
      assert_nil(@msg_iobuf.data)
      @msg_logger.debug('world')
      assert_nil(@msg_iobuf.data)
      @wrn_logger.print(WWWsrv::Logger::DEBUG, 'hello')
      assert_nil(@wrn_iobuf.data)
      @wrn_logger.debug('world')
      assert_nil(@wrn_iobuf.data)
      @alt_logger.print(WWWsrv::Logger::DEBUG, 'hello')
      assert_nil(@alt_iobuf.data)
      @alt_logger.debug('world')
      assert_nil(@alt_iobuf.data)
      @err_logger.print(WWWsrv::Logger::DEBUG, 'hello')
      assert_nil(@err_iobuf.data)
      @err_logger.debug('world')
      assert_nil(@err_iobuf.data)
    end

    def test_system
      @dbg_logger.print(WWWsrv::Logger::SYSTEM, 'hello')
      assert_match(/hello/, @dbg_iobuf.data)
      @dbg_logger.system('world')
      assert_match(/world/, @dbg_iobuf.data)
      @dsc_logger.print(WWWsrv::Logger::SYSTEM, 'hello')
      assert_match(/hello/, @dsc_iobuf.data)
      @dsc_logger.system('world')
      assert_match(/world/, @dsc_iobuf.data)
      @msg_logger.print(WWWsrv::Logger::SYSTEM, 'hello')
      assert_nil(@msg_iobuf.data)
      @msg_logger.system('world')
      assert_nil(@msg_iobuf.data)
      @wrn_logger.print(WWWsrv::Logger::SYSTEM, 'hello')
      assert_nil(@wrn_iobuf.data)
      @wrn_logger.system('world')
      assert_nil(@wrn_iobuf.data)
      @alt_logger.print(WWWsrv::Logger::SYSTEM, 'hello')
      assert_nil(@alt_iobuf.data)
      @alt_logger.system('world')
      assert_nil(@alt_iobuf.data)
      @err_logger.print(WWWsrv::Logger::SYSTEM, 'hello')
      assert_nil(@err_iobuf.data)
      @err_logger.system('world')
      assert_nil(@err_iobuf.data)
    end

    def test_message
      @dbg_logger.print(WWWsrv::Logger::MESSAGE, 'hello')
      assert_match(/hello/, @dbg_iobuf.data)
      @dbg_logger.message('world')
      assert_match(/world/, @dbg_iobuf.data)
      @dsc_logger.print(WWWsrv::Logger::MESSAGE, 'hello')
      assert_match(/hello/, @dsc_iobuf.data)
      @dsc_logger.message('world')
      assert_match(/world/, @dsc_iobuf.data)
      @msg_logger.print(WWWsrv::Logger::MESSAGE, 'hello')
      assert_match(/hello/, @msg_iobuf.data)
      @msg_logger.message('world')
      assert_match(/world/, @msg_iobuf.data)
      @wrn_logger.print(WWWsrv::Logger::MESSAGE, 'hello')
      assert_nil(@wrn_iobuf.data)
      @wrn_logger.message('world')
      assert_nil(@wrn_iobuf.data)
      @alt_logger.print(WWWsrv::Logger::MESSAGE, 'hello')
      assert_nil(@alt_iobuf.data)
      @alt_logger.message('world')
      assert_nil(@alt_iobuf.data)
      @err_logger.print(WWWsrv::Logger::MESSAGE, 'error')
      assert_nil(@err_iobuf.data)
      @err_logger.message('hello')
      assert_nil(@err_iobuf.data)
    end

    def test_warning
      @dbg_logger.print(WWWsrv::Logger::WARNING, 'hello')
      assert_match(/hello/, @dbg_iobuf.data)
      @dbg_logger.warning('world')
      assert_match(/world/, @dbg_iobuf.data)
      @dsc_logger.print(WWWsrv::Logger::WARNING, 'hello')
      assert_match(/hello/, @dsc_iobuf.data)
      @dsc_logger.warning('world')
      assert_match(/world/, @dsc_iobuf.data)
      @msg_logger.print(WWWsrv::Logger::WARNING, 'hello')
      assert_match(/hello/, @msg_iobuf.data)
      @msg_logger.warning('world')
      assert_match(/world/, @msg_iobuf.data)
      @wrn_logger.print(WWWsrv::Logger::WARNING, 'hello')
      assert_match(/hello/, @wrn_iobuf.data)
      @wrn_logger.warning('world')
      assert_match(/world/, @wrn_iobuf.data)
      @alt_logger.print(WWWsrv::Logger::WARNING, 'hello')
      assert_nil(@alt_iobuf.data)
      @alt_logger.warning('world')
      assert_nil(@alt_iobuf.data)
      @err_logger.print(WWWsrv::Logger::WARNING, 'hello')
      assert_nil(@err_iobuf.data)
      @err_logger.warning('world')
      assert_nil(@err_iobuf.data)
    end

    def test_alert
      @dbg_logger.print(WWWsrv::Logger::ALERT, 'hello')
      assert_match(/hello/, @dbg_iobuf.data)
      @dbg_logger.alert('world')
      assert_match(/world/, @dbg_iobuf.data)
      @dsc_logger.print(WWWsrv::Logger::ALERT, 'hello')
      assert_match(/hello/, @dsc_iobuf.data)
      @dsc_logger.alert('world')
      assert_match(/world/, @dsc_iobuf.data)
      @msg_logger.print(WWWsrv::Logger::ALERT, 'hello')
      assert_match(/hello/, @msg_iobuf.data)
      @msg_logger.alert('world')
      assert_match(/world/, @msg_iobuf.data)
      @wrn_logger.print(WWWsrv::Logger::ALERT, 'hello')
      assert_match(/hello/, @wrn_iobuf.data)
      @wrn_logger.alert('world')
      assert_match(/world/, @wrn_iobuf.data)
      @alt_logger.print(WWWsrv::Logger::ALERT, 'hello')
      assert_match(/hello/, @alt_iobuf.data)
      @alt_logger.alert('world')
      assert_match(/world/, @alt_iobuf.data)
      @err_logger.print(WWWsrv::Logger::ALERT, 'hello')
      assert_nil(@err_iobuf.data)
      @err_logger.alert('world')
      assert_nil(@err_iobuf.data)
    end

    def test_error
      @dbg_logger.print(WWWsrv::Logger::ERROR, 'hello')
      assert_match(/hello/, @dbg_iobuf.data)
      @dbg_logger.error('world')
      assert_match(/world/, @dbg_iobuf.data)
      @dsc_logger.print(WWWsrv::Logger::ERROR, 'hello')
      assert_match(/hello/, @dsc_iobuf.data)
      @dsc_logger.error('world')
      assert_match(/world/, @dsc_iobuf.data)
      @msg_logger.print(WWWsrv::Logger::ERROR, 'hello')
      assert_match(/hello/, @msg_iobuf.data)
      @msg_logger.error('world')
      assert_match(/world/, @msg_iobuf.data)
      @wrn_logger.print(WWWsrv::Logger::ERROR, 'hello')
      assert_match(/hello/, @wrn_iobuf.data)
      @wrn_logger.error('world')
      assert_match(/world/, @wrn_iobuf.data)
      @alt_logger.print(WWWsrv::Logger::ERROR, 'hello')
      assert_match(/hello/, @alt_iobuf.data)
      @alt_logger.error('world')
      assert_match(/world/, @alt_iobuf.data)
      @err_logger.print(WWWsrv::Logger::ERROR, 'hello')
      assert_match(/hello/, @err_iobuf.data)
      @err_logger.error('world')
      assert_match(/world/, @err_iobuf.data)
    end
  end

  class TestMultiLogger < RUNIT::TestCase
    def setup
      @dbg_iobuf = PseudoIO::new
      @msg_iobuf = PseudoIO::new
      @logger = WWWsrv::MultiLogger::new
      @logger.add_logger(WWWsrv::Logger::new(@dbg_iobuf, WWWsrv::Logger::DEBUG))
      @logger.add_logger(WWWsrv::Logger::new(@msg_iobuf, WWWsrv::Logger::MESSAGE))
    end

    def test_new
      assert_instance_of(WWWsrv::MultiLogger, @logger)
    end

    def test_message
      @logger.message('Hello world')
      assert_match(/Hello world/, @dbg_iobuf.data)
      assert_match(/Hello world/, @msg_iobuf.data)
    end

    def test_debug
      @logger.debug('Hello world')
      assert_match(/Hello world/, @dbg_iobuf.data)
      assert_nil(@msg_iobuf.data)
    end
  end
end

if ($0 == __FILE__) then
  TestWWWsrv::constants.map{ |name|
    TestWWWsrv::const_get(name)
  }.select{ |const|
    (const.kind_of? Class) && (const < RUNIT::TestCase)
  }.each do |test_case|
    RUNIT::CUI::TestRunner::run(test_case::suite)
  end
end
