#!/usr/local/bin/ruby
# $Id: test_path.rb,v 1.2 2002/01/01 13:21:12 toki Exp $
# -
# Test of WWWsrv::Path module

require 'wwwsrv/path'
require 'runit/testcase'
require 'runit/cui/testrunner'

module TestWWWsrv
  class TestPath < RUNIT::TestCase
    def test_split_path
      path_fragments = WWWsrv::Path::split_path('/test/hello/')
      assert_equal([ '', 'test', 'hello' ], path_fragments)
      path_fragments = WWWsrv::Path::split_path('/test/hello')
      assert_equal([ '', 'test', 'hello' ], path_fragments)
      path_fragments = WWWsrv::Path::split_path('/')
      assert_equal([ '' ], path_fragments)
      path_fragments = WWWsrv::Path::split_path('')
      assert_equal([], path_fragments)
      path_fragments = WWWsrv::Path::split_path('http://localhost')
      assert_equal([ 'http:', '', 'localhost' ], path_fragments)
    end

    def test_join_path
      assert_equal('/hello/world', WWWsrv::Path::join_path([ '', 'hello', 'world' ]))
      assert_equal('/', WWWsrv::Path::join_path([ '' ]))
      assert_equal('http://localhost', WWWsrv::Path::join_path([ 'http:', '', 'localhost' ]))
    end

    def test_subpath
      assert_equal('/hello', WWWsrv::Path::subpath('/', '/hello'))
      assert_equal('/world', WWWsrv::Path::subpath('/hello', '/hello/world'))
      assert_exception(RuntimeError) { WWWsrv::Path::subpath('/hello/world', '/hello') }
      assert_exception(RuntimeError) { WWWsrv::Path::subpath('/hello', '/detarame') }
    end

    def test_normalize_path
      assert_equal('/foo/bar', WWWsrv::Path::normalize_path('/foo/./bar')[0])
      assert_equal('/bar', WWWsrv::Path::normalize_path('/foo/../bar')[0])
      assert_equal('/foo/', WWWsrv::Path::normalize_path('/foo/.')[0])
      assert_equal('/foo/', WWWsrv::Path::normalize_path('/foo/bar/..')[0])
      assert_equal('/foo', WWWsrv::Path::normalize_path('/../foo')[0])
      assert_equal('/foo/', WWWsrv::Path::normalize_path('/foo/.')[0])
      assert_equal('/foo', WWWsrv::Path::normalize_path('/../foo')[0])
      assert_equal('/', WWWsrv::Path::normalize_path('/..')[0])
      assert_equal('/foo', WWWsrv::Path::normalize_path('/foo?bar')[0])
      assert_equal('bar', WWWsrv::Path::normalize_path('/foo?bar')[1])
      assert_equal('/foo bar', WWWsrv::Path::normalize_path('/foo+bar')[0])
      assert_equal('/foo/~bar', WWWsrv::Path::normalize_path('/foo/%7Ebar')[0])
    end
  end
end

if ($0 == __FILE__) then
  TestWWWsrv::constants.map{ |name|
    TestWWWsrv::const_get(name)
  }.select{ |const|
    (const.kind_of? Class) && (const < RUNIT::TestCase)
  }.each do |test_case|
    RUNIT::CUI::TestRunner::run(test_case::suite)
  end
end
