#!/usr/local/bin/ruby
# $Id: test_process.rb,v 1.2 2002/01/01 13:21:13 toki Exp $
# -
# Test of WWWsrv::EnvironmentProtect module

require 'wwwsrv/process'
require 'runit/testcase'
require 'runit/cui/testrunner'

module TestWWWsrv
  class TestEnvironmentProtect < RUNIT::TestCase
    def test_protect_environ
      tmp_env = [
	[ 'MESSAGE', 'Hello world' ],
	[ 'DEBUG', 'True' ]
      ]
      path = ENV['PATH']
      WWWsrv::EnvironmentProtect::protect_environ(tmp_env) do
	for name, value in ENV
	  if (name == 'PATH') then
	    assert_equal(path, value)
	  else
	    assert_not_nil(env_pair = tmp_env.assoc(name))
	    assert_equal(env_pair.last, value)
	  end
	end
      end
    end

    def test_protect_environ_not_preserved
      tmp_env = [
	[ 'MESSAGE', 'Hello world' ],
	[ 'DEBUG', 'True' ]
      ]
      WWWsrv::EnvironmentProtect::protect_environ(tmp_env, nil) do
	for name, value in ENV
	  assert_not_nil(env_pair = tmp_env.assoc(name))
	  assert_equal(env_pair.last, value)
	end
      end
    end

    def test_protect_chdir
      curr_work_dir = Dir::getwd
      root = File::expand_path('/')
      WWWsrv::EnvironmentProtect::protect_chdir(root) do
	assert_equal(root, Dir::getwd)
      end
      assert_equal(curr_work_dir, Dir::getwd)
    end
  end
end

if ($0 == __FILE__) then
  TestWWWsrv::constants.map{ |name|
    TestWWWsrv::const_get(name)
  }.select{ |const|
    (const.kind_of? Class) && (const < RUNIT::TestCase)
  }.each do |test_case|
    RUNIT::CUI::TestRunner::run(test_case::suite)
  end
end
