# $Id: alert.rb,v 1.1.2.1 2002/01/14 13:13:13 toki Exp $
# -
# alert messages

=begin
Copyright (c) 2001-2002
	TOKI Yoshinori <toki@freedom.ne.jp>. All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  1.  Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.

  2.  Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

  3.  The name of the author may not be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.
=end

require 'wwwsrv/const'
require 'wwwsrv/document'

module WWWsrv
  class AlertDocument < Document
    def content(sid, srv_opt, prefix, request, response)
      srv_opt.log.debug("sid #{sid}: debug #{type}")
      response.header.content_type = 'text/html'
      yield(response)
      yield(<<-EOF)
<html>
<head>
<title>#{response.version} #{response.status} #{response.reason}</title>
</head>
<body>
<h1>#{response.version} #{response.status} #{response.reason}</h1>
      EOF

      if (srv_opt.error) then
	yield("<p>#{quoteHTML(srv_opt.error.to_s)}</p>\n")
      else
	yield("<p>#{quoteHTML(response.reason)}</p>\n")
      end

      yield(<<-EOF)
<hr>
<address>#{SRVNAME} version #{VERSION} (#{CODENAME})</address>
</body>
</html>
      EOF

      self
    end
  end

  # for backward compatibility
  Alarm = AlertDocument

  class DebugAlertDocument < Document
    def content(sid, srv_opt, prefix, request, response)
      srv_opt.log.debug("sid #{sid}: debug #{type}")
      response.header.content_type = 'text/html'
      yield(response)
      yield(<<-EOF)
<html>
<head>
<title>#{response.version} #{response.status} #{response.reason}</title>
</head>
<body>
<h1>#{response.version} #{response.status} #{response.reason}</h1>
      EOF

      if (request) then
        yield(<<-EOF)
<h2>Client</h2>
<dl>
<dt>name</dt>
<dd>#{request.client.name}</dd>
<dt>address</dt>
<dd>#{request.client.address}</dd>
</dl>
<h2>Request</h2>
<pre>#{request.line}</pre>
<h2>Header</h2>
<dl>
        EOF

        for field_name, field_value in request.header
	  yield(<<-EOF)
<dt>#{quoteHTML(field_name)}</dt>
<dd>#{quoteHTML(field_value)}</dd>
	  EOF
	end

        yield(<<-EOF)
</dl>
        EOF
      end

      if (srv_opt.error) then
	if (srv_opt.error.kind_of? Exception) then
	  yield(<<-EOF)
<h2>Server Exception</h2>
<dl>
<dt>Exception</dt>
<dd>#{quoteHTML(srv_opt.error.type.to_s)}</dd>
<dt>Message</dt>
<dd>#{quoteHTML(srv_opt.error.message)}</dd>
</dl>
<h2>Server Backtrace</h2>
<ol>
	  EOF

	  for frame in srv_opt.error.backtrace
            yield(<<-EOF)
<li>#{quoteHTML(frame)}</li>
            EOF
	  end

          yield(<<-EOF)
</ol>
          EOF
        else
          yield(<<-EOF)
<h2>Server Message</h2>
<p>#{quoteHTML(srv_opt.error)}</p>
          EOF
        end
      end

      yield(<<-EOF)
<hr>
<address>#{SRVNAME} version #{VERSION} (#{CODENAME})</address>
</body>
</html>
      EOF

      self
    end
  end

  # for backward compatibility
  DebugAlarm = DebugAlertDocument
end
