# $Id: auth.rb,v 1.2.2.1 2002/01/14 13:13:13 toki Exp $
# -
# HTTP authentication

=begin
Copyright (c) 2001-2002
	TOKI Yoshinori <toki@freedom.ne.jp>. All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  1.  Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.

  2.  Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

  3.  The name of the author may not be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.
=end

require 'wwwsrv/const'
require 'wwwsrv/document'

module WWWsrv
  class BasicAuthentication < DocumentModifier
    def initialize(document, realm, passwd)
      super(document)
      @realm = realm
      @passwd = passwd
    end

    def authenticate(header)
      if (credentials = header.authorization) then
	if (credentials =~ /^Basic\s+/ni) then
	  base64_user_pass = credentials.split(/\s+/n, 2).last
	  user_pass = base64_user_pass.unpack('m').first
	  userid, password = user_pass.split(/:/n, 2)
	  if ((userid && ! userid.empty?) && (password && ! password.empty?)) then
	    if (@passwd[userid] == password) then
	      return userid
	    end
	  end
	end
      end

      nil
    end
    private :authenticate

    def modify_request(sid, srv_opt, context, prefix, request, response)
      response.header.www_authenticate = 'Basic realm="' + @realm.gsub(/"/n, '\"') + '"'
      if (request.option.auth_userid = authenticate(request.header)) then
	request.option.auth_type = 'Basic'
	srv_opt.log.system("sid #{sid}: ",
			   "WWW Basic Authentication: allow ",
			   (! prefix.empty?) ? prefix : '/',
			   ': ', @realm, ': ', request.client.to_s)
      else
	srv_opt.log.system("sid #{sid}: ",
			   "WWW Basic Authentication: deny ",
			   (! prefix.empty?) ? prefix : '/',
			   ': ', @realm, ': ', request.client.to_s)
	response.status = 401
	response.header.content_type = 'text/plain'
	yield(response)
	yield("Not authorized.\n")
	terminate
      end

      self
    end
  end

  # for backward compatibility
  Sentinel = BasicAuthentication
end
