# $Id: cgi.rb,v 1.6.2.2 2002/02/27 14:16:52 toki Exp $
# -
# Common Gateway Interface

=begin
Copyright (c) 2001-2002
	TOKI Yoshinori <toki@freedom.ne.jp>. All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  1.  Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.

  2.  Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

  3.  The name of the author may not be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.
=end

require 'wwwsrv/const'
require 'wwwsrv/ioutil'
require 'wwwsrv/chunk'
require 'wwwsrv/process'
require 'wwwsrv/header'
require 'wwwsrv/response'
require 'wwwsrv/document'

module WWWsrv
  class CGIdocument < Document
    include Timeout
    include IOunit
    include EnvironmentProtect

    class << self
      def cgi_var(request, srv_opt)
	cgi_var = {}
	cgi_var['GATEWAY_INTERFACE'] = 'CGI/1.1'
	cgi_var['REQUEST_METHOD'] = request.method
	cgi_var['QUERY_STRING'] = request.query || ''
	cgi_var['REMOTE_ADDR'] = request.client.address
	if (request.client.name != request.client.address) then
	  cgi_var['REMOTE_HOST'] = request.client.name
	end
	cgi_var['PATH_INFO'] = ''
	cgi_var['SCRIPT_NAME'] = request.normalized_path
	cgi_var['SERVER_NAME'] = request.host.name
	cgi_var['SERVER_PORT'] = request.host.port.to_s
	cgi_var['SERVER_PROTOCOL'] = request.version.to_s
	cgi_var['SERVER_SOFTWARE'] = PRODUCT
	if (request.option.auth_userid) then
	  cgi_var['AUTH_TYPE'] = request.option.auth_type
	  cgi_var['REMOTE_USER'] = request.option.auth_userid
	end
	for field_name, field_value in request.header
	  case (field_name)
	  when 'Content-Type'
	    cgi_var['CONTENT_TYPE'] = field_value
	  when 'Content-Length'
	    cgi_var['CONTENT_LENGTH'] = field_value
	  when /Authorization/n
	    # skip!
	  else
	    name = field_name.upcase
	    name.gsub!(/-/n, '_')
	    cgi_var['HTTP_' + name] = field_value
	  end
	end

	cgi_var
      end
    end

    IO_TIMEOUT = 30

    def initialize(base_dir, nph=false, exec=nil)
      super()
      @base_dir = File::expand_path(base_dir)
      @allow = Regexp::compile('^' + Regexp::quote(@base_dir), false, 'n')
      @nph = nph
      @exec = exec
      @env = {}
      @timeout = IO_TIMEOUT
      @arguments_allowed = true
      @arguments_limit_size = 1024
    end

    attr_reader   :env
    attr_accessor :timeout
    attr_accessor :arguments_allowed
    attr_accessor :arguments_limit_size

    alias environ env

    def content(sid, srv_opt, prefix, request, response)
      srv_opt.log.debug("sid #{sid}: debug #{type}")

      cgi_path = request.subpath(prefix)
      cgi_path.gsub!(%r!/!n, File::Separator)
      cgi_path = File::expand_path(File::join(@base_dir, cgi_path))
      if (cgi_path !~ @allow || ! (File::exists? cgi_path)) then
	http_error(404, "Not found a document: #{request.normalized_path}.")
      end
      response.option.local_path = cgi_path
      srv_opt.log.debug("sid #{sid}: debug #{type}: allow access: #{cgi_path}")

      cgi_var = CGIdocument::cgi_var(request, srv_opt)
      for name, value in @env
	cgi_var[name] = value
      end

      cgi_exec = ''
      if (@exec) then
	cgi_exec.concat(@exec) 
	cgi_exec.concat(' ')
      end
      cgi_exec.concat(cgi_path)
      if (@arguments_allowed && request.query && ! request.query.empty? && request.query !~ /=/n) then
	if (request.query.length > @arguments_limit_size) then
	  http_error(414, 'Too long CGI arguments.')
	end
	cgi_exec.concat(' ' + request.query.split(/\+/n).map{ |argument|
			  quoteSHELL(decodeURL(argument))
			}.join(' '))
      end
      flags = 'r'
      flags.concat('+') if (request.method == 'POST' || request.header.content_length)
      cgi_io = LOCK.synchronize{
	protect_environ(cgi_var) {
	  protect_chdir(File::dirname(cgi_path)) {
	    IO::popen(cgi_exec, flags)
	  }
	}
      }
      cgi_io.binmode

      begin
	if (request.method == 'POST' || request.header.content_length) then
	  begin
	    if (transfer_encoding = request.header.transfer_encoding) then
	      if (transfer_encoding.downcase == CHUNKED) then
		request.content.each_chunk do |chunk_data, chunk_extension|
		  wait(@timeout) do
		    if (chunk_data) then
		      cgi_io.printf('%x', chunk_data.length)
		    else
		      cgi_io.write('0')
		    end

		    for chunk_ext_name, chunk_ext_value in chunk_extension
		      cgi_io.write('; ')
		      cgi_io.write(chunk_ext_name)
		      if (chunk_ext_value) then
			cgi_io.write('=')
			cgi_io.write(chunk_ext_value)
		      end
		    end
		    cgi_io.write("\r\n")

		    if (chunk_data) then
		      cgi_io.write(chunk_data)
		      cgi_io.write("\r\n")
		    end
		  end
		end

		wait(@timeout) do
		  for field_name, field_value in request.trailer
		    cgi_io.write(field_name)
		    cgi_io.write(': ')
		    cgi_io.write(field_value)
		    cgi_io.write("\r\n")
		  end
		  cgi_io.write("\r\n")
		end
	      else
		err_msg = "unknown Transfer-Encoding of request body: #{transfer_encoding}"
		http_error(400,  err_msg, { 'Connection' => 'close' })
	      end
	    else
	      while (content_fragment = request.content.read(IO_UNIT_SIZE))
		cgi_io.write(content_fragment)
	      end
	    end
	  ensure
	    cgi_io.close_write
	  end
	end

	if (request.version >= HTTP_1_0) then
	  if (@nph) then
	    status_line = wait(@timeout) { Response::parse_status_line(cgi_io) }
	    unless (status_line) then
	      srv_opt.log.warning("sid #{sid}: #{type}: empty status line: #{cgi_path}")
	      http_error(500, "Empty status line: #{request.normalized_path}.")
	    end
	    http_version, status_code, reason = status_line
	    response.version = http_version
	    response.status = status_code
	    response.reason = reason
	  end

	  begin
	    cgi_header = wait(@timeout) { Header::parse(cgi_io) }
	    def_field = Hash[*response.header.to_a.flatten]
	    for field_name, field_value in cgi_header
	      if (def_field.include? field_name) then
		def_field.delete(field_name)
		response.header[field_name] = nil
	      end
	      response.header.push(field_name, field_value)
	    end
	  rescue ParseError
	    srv_opt.log.warning("sid #{sid}: #{type}: ",
				"failed to parse response header: #{cgi_path}")
	    http_error(500, "Failed to parse response header: #{request.normalized_path}.")
	  end

	  unless (@nph) then
	    if (cgi_header.empty?) then
	      http_error(500, 'Empty CGI response.')
	    end

	    if (response.header.location) then
	      if (response.header.location !~ %r!^[A-Za-z][A-Za-z0-9.+-]*://!n) then
		if (response.header.location !~ %r!^/!n) then
		  response.header.location =
		    request.normalized_path.sub(%r![^/]*$!, '') + response.header.location
		end
		response.header.location =
		  srv_opt.scheme + '://' + request.host.name + ':' + request.host.port.to_s +
		  response.header.location
	      end
	    end

	    if (response.header.status) then
	      status_code, reason = Response::parse_status(response.header.status)
	      response.status = status_code
	      response.reason = reason
	      response.header.status = nil
	    elsif (response.header.location) then
	      response.status = 302
	    end
	  end
	end

	response_transfer_encoding = response.header.transfer_encoding
	response_content_length = response.header.content_length
	yield(response) # response header modified by server

	if (response_transfer_encoding) then
	  if (response_transfer_encoding.downcase == CHUNKED) then
	    chunk_data = chunk_extension = nil
	    while (true)
	      chunk_data, chunk_extension = wait(@timeout) { ChunkedTransferCoding::parse_chunk(cgi_io) }
	      yield(chunk_data, *chunk_extension)
	      chunk_data or break
	    end
	    response.trailer = wait(@timeout) { Header::parse(cgi_io) }
	  else
	    err_msg = "unknown Transfer-Encoding of response body: #{response_transfer_encoding}"
	    http_error(400, err_msg, { 'Connection' => 'close' })
	  end
	elsif (response_content_length) then
	  block_size = io_unit(cgi_io.stat)
	  content_length = response_content_length.to_i
	  content_fragment = nil
	  while (content_length > block_size)
	    content_fragment = wait(@timeout) { cgi_io.read(block_size) }
	    content_fragment or break
	    yield(content_fragment)
	    content_length -= content_fragment.length
	  end

	  if (content_length > 0) then
	    content_fragment = wait(@timeout) { cgi_io.read(content_length) }
	    yield(content_fragment) if content_fragment
	  end
	else
	  block_size = io_unit(cgi_io.stat)
	  content_fragment = nil
	  while (true)
	    content_fragment = wait(@timeout) { cgi_io.read(block_size) }
	    content_fragment or break
	    yield(content_fragment)
	  end
	end
      ensure
	cgi_io.close
      end

      self
    end
  end
end
