# $Id: chunk.rb,v 1.5.2.1 2002/01/14 13:13:14 toki Exp $
# -
# Chunked Transfer Encoding

=begin
Copyright (c) 2001-2002
	TOKI Yoshinori <toki@freedom.ne.jp>. All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  1.  Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.

  2.  Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

  3.  The name of the author may not be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.
=end

require 'wwwsrv/const'
require 'wwwsrv/token'

module WWWsrv
  class ChunkParseError < ParseError; end

  module ChunkedTransferCoding
    def parse_chunk(input)
      chunk_line = input.gets(LF) or return nil
      chunk_line.chomp!(LF)
      chunk_line.chomp!(CR)
      chunk_size, chunk_extension = chunk_line.split(/\s*;\s*/n, 2)
      (chunk_size && chunk_size =~ /^[0-9A-Fa-f]+$/n) or
	raise ChunkParseError, "bad chunk size format: #{chunk_size.inspect}"

      chunk_ext_alist = []
      if (chunk_extension) then
	while (chunk_ext_name = Token::fetch_token(chunk_extension, true))
	  chunk_ext_value = nil
	  if (chunk_extension.sub!(/^=/n, '')) then
	    if (chunk_extension =~ /^"/n) then
	      chunk_ext_value = Token::fetch_quoted(chunk_extension, true)
	    else
	      chunk_ext_value = Token::fetch_token(chunk_extension, true)
	    end
	  end
	  chunk_ext_alist.push([ chunk_ext_name, chunk_ext_value ])
	  chunk_extension.sub!(/\s*(;\s*)?/n, '') or
	    raise ChunkParseError, "bad chunk extension format: #{chunk_extension.inspect}"
	end
      end

      size = chunk_size.hex
      chunk_data = nil
      if (size > 0) then
	chunk_data = input.read(size)
	(chunk_data && chunk_data.length == size) or
	  raise ChunkParseError, 'closed stream'
	chunk_separator = input.gets(LF)
	(chunk_separator == LF || chunk_separator == CRLF) or
	  raise ChunkParseError, "bad chunk separator: #{chunk_separator.dump}"
      end

      return chunk_data, chunk_ext_alist
    end
    module_function :parse_chunk

    def each_chunk(input)
      while (true)
	chunk_data, chunk_extension = parse_chunk(input)
	yield(chunk_data, chunk_extension)
	chunk_data or break
      end

      self
    end
    module_function :each_chunk
  end
end
