# $Id: erbdoc.rb,v 1.2.2.1 2002/01/14 13:13:16 toki Exp $
# -
# eRuby document

=begin
Copyright (c) 2001-2002
	TOKI Yoshinori <toki@freedom.ne.jp>. All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  1.  Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.

  2.  Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

  3.  The name of the author may not be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.
=end

require 'erb/erbl'
require 'wwwsrv/const'
require 'wwwsrv/document'

module WWWsrv
  module ERubyBind
    def eruby_bind(__sid__, __srv_opt__, __request__, __response__)
      Module::new.module_eval{
	const_set(:SID,      __sid__)
	const_set(:SRV_OPT,  __srv_opt__)
	const_set(:REQUEST,  __request__)
	const_set(:RESPONSE, __response__)

	binding
      }
    end
    module_function :eruby_bind
  end

  class ERubyDocument < Document
    include ERubyBind

    def initialize(base_dir, sefe_level=nil, allow_method=%w[ GET HEAD POST ])
      super()
      @base_dir = File::expand_path(base_dir)
      @allow = Regexp::compile('^' + Regexp::quote(@base_dir), false, 'n')
      @sefe_level = sefe_level
      @allow_method = {}
      for name in allow_method
	@allow_method[name] = true
      end
    end

    def content(sid, srv_opt, prefix, request, response)
      srv_opt.log.debug("sid: #{sid}: debug #{type}")

      erb_path = request.reduce_path(prefix)
      erb_path.gsub!(%r!/!n, File::Separator)
      erb_path = File::expand_path(File::join(@base_dir, erb_path))
      unless (erb_path =~ @allow && (File::exists? erb_path)) then
	http_error(404, "Not found a document: #{request.normalized_path}")
      end
      unless (@allow_method.include? request.method) then
	http_error(405, "Not allowed request method: #{request.method}.", { 'Allow' => @allow_method.keys.join(', ') })
      end
      srv_opt.log.debug("sid #{sid}: debug #{type}: allow access: #{erb_path}")

      response.option.local_path = erb_path
      response.header.content_type = 'text/html'

      erb = ERbLight::new(File::open(erb_path) { |input| input.read }, @safe_level)
      content_body = erb.result(eruby_bind(sid, srv_opt, request, response))

      response.header.content_length = content_body.length.to_s
      yield(response)
      yield(content_body)

      self
    end
  end

  class ERubyFilter < DocumentModifier
    include ERubyBind

    def initialize(document, safe_level=nil)
      super(document)
      @safe_level = safe_level
    end

    def modify_request(sid, srv_opt, context, prefix, request, response)
      cond_header = request.header.select{ |field_name, field_value|
	field_name =~ /^If-/ni
      }
      for field_name, field_value in cond_header
	request.header.pop(field_name)
      end
      request.header.range = nil
      context[:request] = request

      self
    end

    def modify_response(sid, srv_opt, context, prefix, request, response)
      context[:response] = response
      context[:content_spool] = ''
      self
    end

    def modify_content(sid, srv_opt, context, content_fragment, *opts)
      context[:content_spool].concat(content_fragment)
      self
    end

    def modify_terminal(sid, srv_opt, context)
      request = context[:request]
      response = context[:response]
      content_spool = context[:content_spool]

      erb = ERbLight::new(content_spool, @safe_level)
      content_body = erb.result(eruby_bind(sid, srv_opt, request, response))

      response.header.etag = nil
      response.header.last_modified = nil
      response.header.content_md5 = nil
      response.header.content_length = content_body.length.to_s
      yield(response)
      yield(content_body)

      self
    end
  end
end
