# $Id: filter.rb,v 1.1.2.1 2002/01/14 13:13:17 toki Exp $
# -
# line oriented document filter

=begin
Copyright (c) 2001-2002
	TOKI Yoshinori <toki@freedom.ne.jp>. All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  1.  Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.

  2.  Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

  3.  The name of the author may not be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.
=end

require 'wwwsrv/const'
require 'wwwsrv/document'

module WWWsrv
  class DocumentFilter < DocumentModifier
    def initialize(document, eol=$/, &filter)
      super(document)
      @line_scan = Regexp::compile('^.*?' + Regexp::quote(eol, 'n'), false, 'n')
      @filter = filter
    end

    def modify_response(sid, srv_opt, context, prefix, request, response)
      response.header.content_length = nil
      yield(response)

      self
    end

    def modify_content(sid, srv_opt, context, content_fragment, *opts)
      if (content_fragment) then
	if (last_fragment = context[:last_fragment]) then
	  content_fragment = last_fragment.concat(content_fragment)
	else
	  content_fragment = content_fragment.clone
	end

	while (content_fragment.sub!(@line_scan, ''))
	  yield(@filter.call($&))
	end

	if (content_fragment && ! content_fragment.empty?) then
	  context[:last_fragment] = content_fragment
	else
	  context[:last_fragment] = nil
	end
      end

      self
    end

    def modify_terminal(sid, srv_opt, context)
      if (last_fragment = context[:last_fragment]) then
	yield(@filter.call(last_fragment))
      end

      self
    end
  end
end
