# $Id: form.rb,v 1.5.2.1 2002/01/14 13:13:17 toki Exp $
# -
# methods of parsing form data

=begin
Copyright (c) 2001-2002
	TOKI Yoshinori <toki@freedom.ne.jp>. All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  1.  Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.

  2.  Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

  3.  The name of the author may not be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.
=end

require 'wwwsrv/const'
require 'wwwsrv/htmlconv'
require 'wwwsrv/multipart'

module WWWsrv
  module FormData
    def parse_query(param_data)
      params = {}
      for param_segment in param_data.split(/&/n)
	param_name, param_value = param_segment.split(/=/n, 2)
	param_name = HTMLconv::quoteHTML(param_name)
	param_value = HTMLconv::quoteHTML(param_value) if param_value
	params[param_name] = [] unless (params.include? param_name)
	params[param_name].push(param_value)
      end

      params
    end
    module_function :parse_query

    def parse_form_data(input, content_type)
      form_data = {}
      multipart = Multipart::parse_multipart(input, content_type)
      for header, body in multipart
	if (header.content_disposition =~ /^form-data/ni) then
	  name_tag, name_value = header.content_disposition.split(/;\s*name=/ni, 2)
	  $& or raise MultipartParseError,
	    "not found a form control name parameter: #{header.content_disposition}"
	  if (name_value =~ /^"/n) then
	    name = Token::fetch_quoted(name_value, true)
	  else
	    name = Token::fetch_token(name_value, true)
	  end
	  name or raise MultipartParseError,
	    "not found a value of a form control name parameter: #{header.content_disposition}"
	  form_data[name] = [] unless (form_data.include? name)
	  form_data[name].push([ header, body ])
	else
	  raise MultipartParseError, 'not found a Content-Disposition header field'
	end
      end

      form_data
    end
    module_function :parse_form_data
  end
end
