# $Id: htmlconv.rb,v 1.1.2.1 2002/01/14 13:13:20 toki Exp $
# -
# HTML data conversion

=begin
Copyright (c) 2001-2002
	TOKI Yoshinori <toki@freedom.ne.jp>. All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  1.  Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.

  2.  Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

  3.  The name of the author may not be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.
=end

require 'parsedate'
require 'wwwsrv/const'

module WWWsrv
  module HTMLconv
    def http_date(time)
      time = time.dup
      time.gmtime
      time.strftime("%a, %d %b %Y %H:%M:%S GMT")
    end
    module_function :http_date

    # for backward compatibility
    def format_date(time)
      http_date(time)
    end
    module_function :format_date

    def quoteHTML(data)
      data = data.dup
      data.gsub!(/&/n, '&amp;')
      data.gsub!(/</n, '&lt;')
      data.gsub!(/>/n, '&gt;')
      data.gsub!(/"/n, '&quot;')

      data
    end
    module_function :quoteHTML

    def unquoteHTML(data)
      data = data.dup
      data.gsub!(/&quot;/n, '"')
      data.gsub!(/&gt;/n, '>')
      data.gsub!(/&lt;/n, '<')
      data.gsub!(/&amp;/n, '&')

      data
    end
    module_function :unquoteHTML

    def quoteSHELL(data)
      # for Bourne shell
      '"' + data.gsub(/["\$\`\\]/n) { |special| "\\" + special } + '"'
    end
    module_function :quoteSHELL

    def encodeURL(data)
      data = data.dup
      data.gsub!(/[^ _0-9A-Za-z.-]/n) { |special| format('%%%02X', special[0]) }
      data.gsub!(/ /n, '+')

      data
    end
    module_function :encodeURL

    def decodeURL(data)
      data = data.dup
      data.gsub!(/\+/n, ' ')
      data.gsub!(/%([0-9A-Fa-f][0-9A-Fa-f])/n) { |special| $1.hex.chr }

      data
    end
    module_function :decodeURL
  end
end
