# $Id: local.rb,v 1.4.2.1 2002/01/14 13:13:20 toki Exp $
# -
# local file document

=begin
Copyright (c) 2001-2002
	TOKI Yoshinori <toki@freedom.ne.jp>. All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  1.  Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.

  2.  Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

  3.  The name of the author may not be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.
=end

require 'md5'
require 'wwwsrv/const'
require 'wwwsrv/path'
require 'wwwsrv/htmlconv'
require 'wwwsrv/ioutil'
require 'wwwsrv/document'

module WWWsrv
  class MIMEresolver
    include Enumerable

    def initialize(default='application/octet-stream')
      @default = default
      @mime_map = {}
    end

    def clone
      new_mime = super
      new_mime.instance_eval do
	@mime_map = {}
      end

      for suffix, mime_type in self
	new_mime.add_mime_type(suffix, mime_type)
      end

      new_mime
    end

    attr_accessor :default

    def add_mime_type(suffix, mime_type)
      mime_map = @mime_map
      suffix_key = suffix.downcase
      suffix_key.reverse!
      suffix_key.each_byte do |ch|
	mime_map[ch] = {} unless (mime_map.include? ch)
	mime_map = mime_map[ch]
      end
      mime_map[:mime_type] = mime_type

      self
    end

    def resolve(path)
      mime_map = @mime_map
      path_key = path.downcase
      path_key.reverse!
      path_key.each_byte do |ch|
	if (next_map = mime_map[ch]) then
	  mime_map = next_map
	elsif (mime_type = mime_map[:mime_type]) then
	  return mime_type
	else
	  break
	end
      end

      @default
    end

    def each
      traverse(@mime_map) do |suffix, mime_type|
	yield(suffix, mime_type)
      end
    end

    def traverse(mime_map, suffix_stack=[])
      if (mime_type = mime_map[:mime_type]) then
	suffix = suffix_stack.map{ |ch| ch.chr }.join
	suffix.reverse!
	yield(suffix, mime_type)
      end

      mime_map.keys.reject{ |ch| ch == :mime_type }.sort.each do |ch|
	suffix_stack.push(ch)
	traverse(mime_map[ch], suffix_stack) do |suffix, mime_type|
	  yield(suffix, mime_type)
	end
	suffix_stack.pop
      end

      self
    end
    private :traverse
  end

  MIME_TYPE = MIMEresolver::new

  [ [ '.txt',     'text/plain' ],
    [ '.text',    'text/plain' ],
    [ '.log',     'text/plain' ],
    [ '.rb',      'text/plain' ],
    [ '.sh',      'text/plain' ],
    [ '.tex',     'text/plain' ],
    [ '.texi',    'text/plain' ],
    [ '.texinfo', 'text/plain' ],
    [ '.htm',     'text/html' ],
    [ '.html',    'text/html' ],
    [ '.shtml',   'text/html' ],
    [ '.sgml',    'text/sgml' ],
    [ '.xml',     'text/xml' ],
    [ '.css',     'text/css' ],
    [ '.jpg',     'image/jpeg' ],
    [ '.jpeg',    'image/jpeg' ],
    [ '.gif',     'image/gif' ],
    [ '.tif',     'image/tiff' ],
    [ '.tiff',    'image/tiff' ],
    [ '.png',     'image/png' ],
    [ '.mp3',     'audio/mpeg' ],
    [ '.mpg',     'video/mpeg' ],
    [ '.mpeg',    'video/mpeg' ],
    [ '.mov',     'video/quicktime' ],
    [ '.rtf',     'application/rtf' ],
    [ '.ps',      'application/postscript' ],
    [ '.eps',     'application/postscript' ],
    [ '.pdf',     'application/pdf' ],
    [ '.doc',     'application/msword' ],
    [ '.xsl',     'application/vnd.ms-excel' ],
    [ '.ppt',     'application/vnd.ms-powerpoint' ],
    [ '.mpp',     'application/vnd.ms-project' ]
  ].each do |suffix, mime_type|
    MIME_TYPE.add_mime_type(suffix, mime_type)
  end

  class LocalDirIndex
    include HTMLconv

    REJECT_FILTER = [
      /^RCS$/n,
      /^CVS$/n,
      /,v$/n,
      /~$/n,
      /^\#.*\#$/n,
      /^\.[^.]/n
    ]

    def initialize(reject_filter=REJECT_FILTER, &comp)
      @reject_filter = reject_filter
      @comp = comp || proc{ |(name_a, stat_a), (name_b, stat_b)|
	cmp = name_a <=> name_b
	cmp = stat_a.mtime <=> stat_b.mtime if (cmp == 0)
	cmp
      }
    end

    def comma(decimal)
      decimal = decimal.to_s
      decimal.reverse!
      decimal = decimal.scan(/\d\d?\d?/n).join(',')
      decimal.reverse!

      decimal
    end
    private :comma

    def browse(sid, srv_opt, prefix, request, response, local_dir)
      response.header.content_type = 'text/html'
      yield(response)

      http_dir = request.normalized_path
      http_dir = http_dir.chomp('/') if (http_dir != '/')
      http_dir = quoteHTML(http_dir)
      yield(<<-EOF)
<html>
<head>
<title>Directory Index of #{http_dir}</title>
</head>
<body>
<h1>Directory Index of #{http_dir}</h1>
<pre>
      EOF

      dirs = []
      files = []
      Dir::foreach(local_dir) do |name|
	catch(:next_entry) do
	  for pattern in @reject_filter
	    (name =~ pattern) and throw(:next_entry)
	  end

	  path_stat = File::stat(File::join(local_dir, name))
	  if (path_stat.ftype == 'directory') then
	    dirs.push([ name, path_stat ])
	  else
	    files.push([ name, path_stat ])
	  end
	end
      end

      dirs.sort!(&@comp)
      files.sort!(&@comp)

      dirs.map!{ |name, stat|
	[ name, stat.mtime.to_s ]
      }
      files.map!{ |name, stat|
	[ name, stat.mtime.to_s, comma(stat.size) ]
      }

      width = [
	(dirs + files).map{ |name, last_modified, *args| name.length }.max,
	(dirs + files).map{ |name, last_modified, *args| last_modified.length }.max,
	files.map{ |name, last_modified, bytes| bytes.length }.max
      ]

      for name, last_modified in dirs
	href = encodeURL(name)
	yield(%Q'[DIR]  <a href="#{href}/">#{name}/</a>  ')
	yield('.' * (width[0] - name.length + 4))
	yield('  ')
	yield(last_modified)
	yield("\n")
      end

      for name, last_modified, bytes in files
	href = encodeURL(name)
	yield(%Q'[FILE] <a href="#{href}">#{name}</a>  ')
	yield('.' * (width[0] - name.length + 5))
	yield('  ')
	yield(last_modified)
	yield(' ' * (width[1] - last_modified.length + 2))
	yield(format("%#{width[2]}s bytes\n", bytes))
      end

      yield(<<-EOF)
</pre>
<hr>
<address>
#{SRVNAME} version #{VERSION} (#{CODENAME})<br>
Powered by <a href="http://www.ruby-lang.org/"
>Ruby</a>
</address>
</body>
</html>
      EOF
    end
  end

  class LocalFileDocument < Document
    include Path
    include IOunit

    INDEX_LIST = [
      'index.html',
      'index.htm'
    ]

    def initialize(root, mime_type=MIME_TYPE, dir_index=LocalDirIndex::new, index_list=INDEX_LIST)
      super()
      @root = File::expand_path(root)
      @allow = Regexp::compile('^' + Regexp::quote(@root), false, 'n')
      @mime_type = mime_type
      @dir_index = dir_index
      @index_list = index_list
    end

    def expand_local(prefix, request)
      path = request.subpath(prefix)
      path.gsub!(%r!/!n, File::Separator)
      if (request.normalized_path =~ %r!/$!n) then
	if (path.empty?) then
	  return @root
	else
	  return File::expand_path(File::join(@root, path))
	end
      else
	if (path.empty?) then
	  return @root
	else
	  return File::expand_path(File::join(@root, path))
	end
      end
    end
    private :expand_local

    def search_index(local_dir)
      for index_name in @index_list
	index_path = File::join(local_dir, index_name)
	if (File::exists? index_path) then
	  return index_path
	end
      end

      nil
    end
    private :search_index

    def entity_tag(stat, head_block)
      md5 = MD5::new
      md5.update(head_block) if head_block
      format('"%x-%x-%s"', stat.size, stat.mtime.to_i, md5.hexdigest)
    end
    private :entity_tag

    def content(sid, srv_opt, prefix, request, response)
      srv_opt.log.debug("sid #{sid}: debug #{type}")
      local_path = expand_local(prefix, request)
      srv_opt.log.debug("sid #{sid}: debug #{type}: local document path: #{local_path}")
      if (local_path !~ @allow) then
	http_error(404, "Not found a document: #{request.normalized_path}.")
      end
      srv_opt.log.debug("sid #{sid}: debug #{type}: allow access: #{local_path}")

      case (request.method)
      when 'GET', 'HEAD'
	if (request.normalized_path =~ %r!/$!n) then
	  if (index_path = search_index(local_path)) then
	    local_path = index_path
	  elsif (! @dir_index) then
	    local_path = File::join(local_path, @index_list.first)
	  end
	end

	if ((File::file? local_path) && (File::readable? local_path)) then
	  File::open(local_path) do |file|
	    file.binmode
	    stat = file.stat
	    block_size = io_unit(stat, Server::BUFFER_SIZE)
	    head_block = file.read(block_size)
	    etag = entity_tag(stat, head_block)
	    response.option.local_path = local_path
	    response.status = request.condition(stat.mtime, etag)
	    response.status = 200 if (response.status == 206) # If-Range not supported
	    response.header.etag = etag
	    response.header.last_modified = http_date(stat.mtime)
	    if (response.status == 200) then
	      response.header.content_type = @mime_type.resolve(local_path)
	      response.header.content_length = stat.size.to_s
	    end
	    status = response.status
	    yield(response)
	    if (request.method == 'GET' && status == 200) then
	      if (head_block) then
		yield(head_block)
		while (content_fragment = file.read(block_size))
		  yield(content_fragment)
		end
	      end
	    end
	  end
	elsif ((File::directory? local_path) && (File::readable? local_path)) then
	  if (request.normalized_path =~ %r!/$!n) then
	    if (@dir_index) then
	      @dir_index.browse(sid, srv_opt, prefix, request, response, local_path) do |arg, *opts|
		yield(arg, *opts)
	      end
	    else
	      http_error(404, "Not found a document: #{request.normalized_path}.")
	    end
	  else
	    response.status = 301
	    response.header.location =
	      srv_opt.scheme + '://' + request.host.name + ':' + request.host.port.to_s +
	      request.normalized_path + '/'
	    response.header.content_type = 'text/plain'
	    location = response.header.location
	    yield(response)
	    yield("Moved to #{location}.\n")
	  end
	else
	  http_error(404, "Not found a document: #{request.normalized_path}.")
	end
      else
	http_error(405, "Not allowed request method: #{request.method}.", { 'Allow' => 'GET, HEAD' })
      end

      self
    end
  end
end
