# $Id: multipart.rb,v 1.5.2.2 2002/02/27 14:17:04 toki Exp $
# -
# methods of parsing multipart

=begin
Copyright (c) 2001-2002
	TOKI Yoshinori <toki@freedom.ne.jp>. All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  1.  Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.

  2.  Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

  3.  The name of the author may not be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.
=end

require 'wwwsrv/const'
require 'wwwsrv/token'
require 'wwwsrv/header'

module WWWsrv
  class MultipartParseError < ParseError; end

  module Multipart
    def each_multipart(input, content_type)
      boundary_tag, boundary_value = content_type.split(/;\s*boundary=/ni, 2)
      $& or raise MultipartParseError,
	"not found a boundary parameter: #{content_type.inspect}"
      if (boundary_value =~ /^"/n) then
	boundary = Token::fetch_quoted(boundary_value, true)
      else
	boundary = Token::fetch_token(boundary_value, true)
      end
      name or raise MultipartParseError,
	"not found a value of a boundary parameter: #{content_type.inspect}"

      separator = '--' + boundary
      terminator = separator + '--'

      while (line = input.gets(LF))
	(line.strip == separator) and break
      end

      catch(:multipart_terminate) do
	loop do
	  catch(:multipart_separate) do
	    yield(:begin, nil)
	    part_header = Header::parse(input)
	    yield(:header, part_header)

	    content_type = part_header.content_type
	    if (content_type && content_type =~ %r!^multipart/\S+!ni) then
	      each_multipart(input, content_type) do |tag, value|
		yield(tag, value)
	      end
	      while (line = input.gets(LF))
		line.strip!
		case (line)
		when separator
		  yield(:end, nil)
		  throw(:multipart_separate)
		when terminator
		  yield(:end, nil)
		  throw(:multipart_terminate)
		end
	      end
	    else
	      prev_line = nil
	      while (line = input.gets(LF))
		case (line.strip)
		when separator
		  if (prev_line) then
		    prev_line.chomp!(LF)
		    prev_line.chomp!(CR)
		  end
		  yield(:body, prev_line) if (prev_line && ! prev_line.empty?)
		  yield(:end, nil)
		  throw(:multipart_separate)
		when terminator
		  if (prev_line) then
		    prev_line.chomp!(LF)
		    prev_line.chomp!(CR)
		  end
		  yield(:body, prev_line) if (prev_line && ! prev_line.empty?)
		  yield(:end, nil)
		  throw(:multipart_terminate)
		else
		  yield(:body, prev_line) if (prev_line && ! prev_line.empty?)
		end
		prev_line = line
	      end
	    end

	    raise MultipartParseError, "not terminated by boundary"
	  end
	end
      end

      self
    end
    module_function :each_multipart

    def parse_multipart(input, content_type)
      curr_part = nil
      multipart_alist = []
      stack = [ [ nil, multipart_alist ] ]
      each_multipart(input, content_type) do |tag, value|
	case (tag)
	when :begin
	  body = nil
	  curr_part = [ nil, nil ]
	  stack.last[1].push(curr_part)
	  stack.push(curr_part)
	when :header
	  curr_part[0] = value
	  if (value.content_type && value.content_type =~ %r!^multipart/\S+!ni) then
	    curr_part[1] = []
	  else
	    curr_part[1] = ''
	  end
	when :body
	  curr_part[1].concat(value)
	when :end
	  stack.pop
	  curr_part = nil
	end
      end

      multipart_alist
    end
    module_function :parse_multipart
  end
end
