# $Id: path.rb,v 1.1.2.1 2002/01/14 13:13:22 toki Exp $
# -
# HTTP path utilities

=begin
Copyright (c) 2001-2002
	TOKI Yoshinori <toki@freedom.ne.jp>. All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  1.  Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.

  2.  Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

  3.  The name of the author may not be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.
=end

require 'wwwsrv/const'

module WWWsrv
  module Path
    def split_path(path)
      path_fragments = path.split(%r!/!n, -1)
      path_fragments.pop if path_fragments.last.empty?
      path_fragments
    end
    module_function :split_path

    def join_path(path_fragments)
      if (path_fragments.length == 1 && path_fragments.first.empty?) then
	return '/'
      else
	return path_fragments.join('/')
      end
    end
    module_function :join_path

    def subpath(prefix, path)
      prefix.empty? and return path
      prefix_fragments = split_path(prefix)
      path_fragments = split_path(path)

      while (! prefix_fragments.empty? && ! path_fragments.empty? && prefix_fragments.first == path_fragments.first)
	prefix_fragments.shift
	path_fragments.shift
      end

      unless (prefix_fragments.empty?) then
	raise "failed to match prefix to path: #{prefix.inspect} vs #{path.inspect}"
      end

      path_fragments.unshift('')
      join_path(path_fragments)
    end
    module_function :subpath

    # for backward compatibility
    alias reduce_path subpath
    module_function :reduce_path

    def normalize_path(path)
      normalized_path, query = path.split(/\?/n, 2)
      if (normalized_path.empty?) then
	normalized_path = '/'
      else
	# expand URL encoding
	normalized_path.gsub!(/\+/n, ' ')
	normalized_path.gsub!(/%([0-9A-Fa-f][0-9A-Fa-f])/n) { |fragment| $1.hex.chr }

	# expand path (from shttpsrv)
	while (normalized_path.gsub!(%r!/\./!n, '/'))
	  # foo/./bar => foo/bar
	end
	while (normalized_path.gsub!(%r![^/]+/\.\./!n, ''))
	  # foo/../bar => bar
	end
	while (normalized_path.gsub!(%r![^/]+/\.\.$!n, ''))
	  # foo/bar/.. => foo/
	end
	while (normalized_path.gsub!(%r!^/\.\./!n, '/'))
	  # /../foo => /foo
	end
	normalized_path.sub!(%r!/\.$!n, '/')  # foo/. => foo/
	normalized_path.sub!(%r!^/..$!n, '/') # /.. => /
      end

      return normalized_path, query
    end
    module_function :normalize_path
  end
end
