# $Id: response.rb,v 1.3.2.2 2002/02/27 14:17:05 toki Exp $
# -
# HTTP response

=begin
Copyright (c) 2001-2002
	TOKI Yoshinori <toki@freedom.ne.jp>. All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  1.  Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.

  2.  Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

  3.  The name of the author may not be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.
=end

require 'wwwsrv/const'
require 'wwwsrv/version'
require 'wwwsrv/header'
require 'wwwsrv/srvopt'

module WWWsrv
  HTTP_MSG = {
    100 => 'Continue',
    101 => 'Switching Protocols',
    200 => 'OK',
    201 => 'Created',
    202 => 'Accepted',
    203 => 'Non-Authoritative Information',
    204 => 'No Content',
    205 => 'Reset Content',
    206 => 'Partial Content',
    300 => 'Multiple Choices',
    301 => 'Moved Permanently',
    302 => 'Found',
    303 => 'See Other',
    304 => 'Not Modified',
    305 => 'Use Proxy',
    307 => 'Temporary Redirect',
    400 => 'Bad Request',
    401 => 'Unauthorized',
    402 => 'Payment Required',
    403 => 'Forbidden',
    404 => 'Not Found',
    405 => 'Method Not Allowed',
    406 => 'Not Acceptable',
    407 => 'Proxy Authentication Required',
    408 => 'Request Timeout',
    409 => 'Conflict',
    410 => 'Gone',
    411 => 'Length Required',
    412 => 'Precondition Failed',
    413 => 'Request Entity Too Large',
    414 => 'Request-URI Too Long',
    415 => 'Unsupported Media Type',
    416 => 'Requested Range Not Satisfiable',
    417 => 'Expectation Failed',
    500 => 'Internal Server Error',
    501 => 'Not Implemented',
    502 => 'Bad Gateway',
    503 => 'Service Unavailable',
    504 => 'Gateway Timeout',
    505 => 'HTTP Version Not Supported'
  }

  class StatusLineParseError < ParseError; end

  class Response
    def initialize(version=HTTP_1_1, status=200, header=Header::new, trailer=Header::new)
      @version = version
      self.status = status
      @header = header
      @trailer = trailer
      @option = Accessor::wrap(Hash::new)
    end

    attr_accessor :version
    attr_accessor :status
    attr_accessor :reason
    attr_accessor :header
    attr_accessor :trailer
    attr_reader   :option

    def status=(new_status)
      @status = new_status
      @reason = HTTP_MSG[@status]

      @status
    end

    def line
      "#{@version} #{@status} #{@reason}"
    end

    # for backward compatibility
    def local_path
      @option.local_path
    end

    # for backward compatibility
    def local_path=(new_path)
      @option.local_path = new_path
    end

    def self::parse_status(field_value)
      status, reason = field_value.split(/\s+/n, 2)

      (! status || status.empty? || status !~ /^[0-9][0-9][0-9]$/n) and
	raise StatusLineParseError, 'failed to parse response status code'
      status_code = status.to_i
      (100 <= status_code && status_code < 600) or
	raise StatusLineParseError, "out of range of status code: #{status_code}"
      (! reason || reason.empty?) and
	raise StatusLineParseError, "failed to parse response reason: #{reason.inspect}"

      return status_code, reason
    end

    def self::parse_status_line(input)
      while (status_line = input.gets(LF))
	status_line.chomp!(LF)
	status_line.chomp!(CR)
	status_line.empty? and next

	version, others = status_line.split(/\s+/n, 2)
	version or raise StatusLineParseError, "not found a HTTP version: #{status_line}"
	http_version = HTTPversion::parse(version)
	status_code, reason = parse_status(others)

	return http_version, status_code, reason
      end

      nil
    end
  end
end
